/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.Assert;
import org.ow2.petals.cli.base.junit.shell.DummyShell;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;

public abstract class BcGatewayAbstractCommandTest extends Assert {

    @Rule
    public final PetalsAdministrationApi adminApi = new PetalsAdministrationApi();

    @Rule
    public final PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule();

    @Rule
    public final ExpectedException exceptions = ExpectedException.none();

    /**
     * Must be filled by the actual test using {@link Before}.
     */
    protected Command command;

    /**
     * Test that the command usage is defined
     */
    @Test
    public void usage() {
        command.setShell(new DummyShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command against argument management, where:
     * <ul>
     * <li>no connection is previously established.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing connection.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void noConnection() throws Exception {

        // Set the environment
        final StringStreamShell shell = new StringStreamShell();

        // Execute command
        command.setShell(shell);
        command.execute(new String[] {});
    }

    /**
     * <p>
     * Test of the command against argument management, where:
     * <ul>
     * <li>an unrecognized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws Exception {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);

        exceptions.expect(CommandBadArgumentNumberException.class);
        exceptions.expectMessage(Matchers
                .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));

        command.execute(new String[] { "--aunesuteruse" });
    }

    /**
     * <p>
     * Test of the command against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-n' is given but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void componentIdentifierValueMissing() throws Exception {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);

        exceptions.expect(CommandMissingArgumentException.class);
        exceptions.expectMessage(Matchers.startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
        exceptions.expectMessage(Matchers.endsWith(BcGatewayAbstractCommand.COMPONENT_ID_SHORT_OPTION));

        try {
            command.execute(new String[] { "-" + BcGatewayAbstractCommand.COMPONENT_ID_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertEquals(BcGatewayAbstractCommand.COMPONENT_ID_SHORT_OPTION, e.getOption().getOpt());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-n' is missing but a component identifier is given</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void componentIdentifierOptionMissing() throws Exception {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);

        exceptions.expect(CommandBadArgumentNumberException.class);
        exceptions.expectMessage(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS);

        command.execute(new String[] { "a-component-id" });
    }
}
