/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway.mock;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.binding.gateway.clientserver.api.AdminRuntimeService;

/**
 * A mock of {@link AdminRuntimeService} for unit testing
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class PetalsBcGatewayRuntimeConfigurationMock implements AdminRuntimeService {

    private Map<String, Integer> transportListeners = new HashMap<>();

    private Map<String, Set<String>> refreshInvoked = new HashMap<>();

    private Map<String, Map<String, Boolean>> reconnectInvoked = new HashMap<>();

    /**
     * was it invoked at all on the component?
     */
    public boolean wasRefreshPropagationsInvoked() {
        return !this.refreshInvoked.isEmpty();
    }

    /**
     * Use <code>null</code> to denote any SU or consumer domain.
     */
    public boolean wasRefreshPropagationsInvoked(final String suName, final String consumerDomain) {
        // we use the null key/element to denote all SU / consumer domain
        return this.refreshInvoked.containsKey(suName)
                && (suName == null || this.refreshInvoked.get(suName).contains(consumerDomain));
    }

    /**
     * was it invoked at all on the component?
     */
    public boolean wasReconnectInvoked() {
        return !this.reconnectInvoked.isEmpty();
    }

    /**
     * Use <code>null</code> to denote any SU, provider domain or force value.
     */
    public boolean wasReconnectInvoked(final String suName, final String providerDomain, final Boolean force) {
        // we use the null key/element to denote all SU / provider domain
        final Map<String, Boolean> pd2force = this.reconnectInvoked.get(suName);
        if (pd2force != null) {
            final Boolean fv = suName == null ? pd2force.get(null) : pd2force.get(providerDomain);
            return fv != null && (force == null || fv == force);
        } else {
            return false;
        }
    }

    @Override
    public void addTransportListener(final String id, final int port) throws PetalsException {
        this.transportListeners.put(id, port);
    }

    @Override
    public void setTransportListenerPort(final String id, final int port) throws PetalsException {
        this.transportListeners.put(id, port);
    }

    @Override
    public Map<String, Object[]> getTransportListeners() {
        final Map<String, Object[]> res = new HashMap<>();
        for (final Entry<String, Integer> e : this.transportListeners.entrySet()) {
            res.put(e.getKey(), new Object[] { e.getValue(), true, "" });
        }
        return res;
    }

    @Override
    public Boolean removeTransportListener(final String id) throws PetalsException {
        return this.transportListeners.remove(id) != null;
    }

    @Override
    public void refreshPropagations() throws PetalsException {
        this.refreshInvoked.put(null, null);
    }

    private Set<String> getOrCreateRP(final String suName) {
        if (!this.refreshInvoked.containsKey(suName)) {
            this.refreshInvoked.put(suName, new HashSet<String>());
        }
        return this.refreshInvoked.get(suName);
    }

    @Override
    public void refreshPropagations(final String suName) throws PetalsException {
        assert suName != null;
        getOrCreateRP(suName).add(null);
    }

    @Override
    public void refreshPropagations(final String suName, final String consumerDomain) throws PetalsException {
        assert suName != null;
        assert consumerDomain != null;
        getOrCreateRP(suName).add(consumerDomain);
    }

    private Map<String, Boolean> getOrCreateRD(final String suName) {
        if (!this.reconnectInvoked.containsKey(suName)) {
            this.reconnectInvoked.put(suName, new HashMap<String, Boolean>());
        }
        return this.reconnectInvoked.get(suName);
    }

    @Override
    public void reconnectDomains(final boolean force) throws PetalsException {
        getOrCreateRD(null).put(null, force);
    }

    @Override
    public void reconnectDomains(final String suName, final boolean force) throws PetalsException {
        assert suName != null;
        getOrCreateRD(suName).put(null, force);
    }

    @Override
    public void reconnectDomains(final String suName, final String providerDomain, final boolean force)
            throws PetalsException {
        assert suName != null;
        assert providerDomain != null;
        getOrCreateRD(suName).put(providerDomain, force);
    }
}
